@@ -52,7 +52,7 @@ const struct datatype fib_addr_type = {
 	.sym_tbl	= &addrtype_tbl,
 };
 
-static const char *fib_result_str(enum nft_fib_result result)
+const char *fib_result_str(enum nft_fib_result result)
 {
 	if (result <= NFT_FIB_RESULT_MAX)
 		return fib_result[result];
@@ -101,10 +101,11 @@ static void fib_expr_clone(struct expr *new, const struct expr *expr)
 	new->fib.flags= expr->fib.flags;
 }
 
-static const struct expr_ops fib_expr_ops = {
+const struct expr_ops fib_expr_ops = {
 	.type		= EXPR_FIB,
 	.name		= "fib",
 	.print		= fib_expr_print,
+	.json		= fib_expr_json,
 	.cmp		= fib_expr_cmp,
 	.clone		= fib_expr_clone,
 };
@@ -134,7 +135,7 @@ struct expr *fib_expr_alloc(const struct location *loc,
 	if (flags & NFTA_FIB_F_PRESENT)
 		type = &boolean_type;
 
-	expr = expr_alloc(loc, &fib_expr_ops, type,
+	expr = expr_alloc(loc, EXPR_FIB, type,
 			  BYTEORDER_HOST_ENDIAN, len);
 
 	expr->fib.result = result;
